unit Properties;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, About, Registry, Buttons, ComCtrls, Spin, ToolIntf, IniFiles;

type
  TfrmProperties = class(TForm)
    btnBuild: TButton;
    btnCancel: TButton;
    btnAbout: TButton;
    tabVersionInfo: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    lblCompiler: TLabel;
    txtCompiler: TEdit;
    btnBrowse: TButton;
    lblResourceName: TLabel;
    txtResource: TEdit;
    lblFileVersion: TLabel;
    lblProductVersion: TLabel;
    lblFileType: TLabel;
    lblFileFlags: TLabel;
    lblOriginalName: TLabel;
    lblInternalName: TLabel;
    lblDescription: TLabel;
    lblProductName: TLabel;
    lblOS: TLabel;
    lblLanguage: TLabel;
    lblCharSet: TLabel;
    txtFileMaj1: TSpinEdit;
    txtFileMin2: TSpinEdit;
    txtFileMaj2: TSpinEdit;
    txtFileMin1: TSpinEdit;
    txtProductMin2: TSpinEdit;
    txtProductMin1: TSpinEdit;
    txtProductMaj1: TSpinEdit;
    txtProductMaj2: TSpinEdit;
    cmbFileType: TComboBox;
    txtFileFlags: TEdit;
    txtOriginalName: TEdit;
    txtInternalName: TEdit;
    txtDescription: TEdit;
    txtProductName: TEdit;
    cmbCharSet: TComboBox;
    cmbLanguage: TComboBox;
    lblCompanyName: TLabel;
    lblCopyright: TLabel;
    lblTradeMark: TLabel;
    lblAuthor: TLabel;
    radDebugBuild: TRadioButton;
    radReleaseBuild: TRadioButton;
    chkSpecialBuild: TCheckBox;
    chkInternalBuild: TCheckBox;
    txtCompanyName: TEdit;
    txtCopyright: TEdit;
    txtTradeMark: TEdit;
    txtAuthor: TEdit;
    cmbOSType: TComboBox;
    chkIncludeDateStamp: TCheckBox;
    btnSave: TButton;
    lblComments: TLabel;
    txtComments: TEdit;
    dlgOpen: TOpenDialog;
    btnLoadValues: TButton;
    chkAutoUpdateSaving: TCheckBox;
    chkAutoUpdateBuilding: TCheckBox;
    procedure btnAboutClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnBuildClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
    procedure btnLoadValuesClick(Sender: TObject);
  public
    TS: TIToolServices;
    Project: String;
    procedure RegisterToolServices(ts: TIToolServices);
    procedure LoadIni(frm: TfrmProperties);
  end;

implementation

{$R *.DFM}

procedure TfrmProperties.LoadIni(frm: TfrmProperties);
var       reg: TRegistry;
begin
  reg:=TRegistry.Create;
  with reg, frm do begin
    try
      RootKey:=HKEY_CURRENT_USER;
      OpenKey('\Software\Abacus Research AG\VerExp', TRUE);
      try
        try
          txtCompiler.Text:=ReadString('Compiler');
        except
        end;
        try
          txtResource.Text:=ReadString('ResourceName');
        except
        end;
        try
          txtCompanyName.Text:=ReadString('CompanyName');
        except
        end;
        try
          txtTradeMark.Text:=ReadString('TradeMark');
        except
        end;
        try
          txtAuthor.Text:=ReadString('Author');
        except
        end;
        try
          cmbOSType.ItemIndex:=ReadInteger('OSType');
        except
          cmbOSType.ItemIndex:=-1;
        end;
        try
          cmbLanguage.ItemIndex:=ReadInteger('Language');
        except
          cmbLanguage.ItemIndex:=-1;
        end;
        try
          cmbCharSet.ItemIndex:=ReadInteger('CharSet');
        except
          cmbCharSet.ItemIndex:=-1;
        end;
        with TIniFile.Create(ChangeFileExt(TS.GetProjectName, '.DOF')) do begin
          txtResource.Text:=ReadString('VerExp', 'RCFile', 'VERINFO.RC');
          chkAutoUpdateBuilding.Checked:=(ReadInteger('VerExp', 'AutoUpdate', 0)<>0);
          chkAutoUpdateSaving.Checked:=(ReadInteger('VerExp', 'AutoUpdateSave', 0)<>0);
          Free;
        end;
      finally
        CloseKey;
      end;
    finally
      reg.Free;
    end;
  end;
end;

procedure TfrmProperties.RegisterToolServices(ts: TIToolServices);
begin
  Self.TS:=ts;
end;

procedure TfrmProperties.btnAboutClick(Sender: TObject);
begin
  ShowAbout;
end;

procedure TfrmProperties.FormCreate(Sender: TObject);
var       s:   String;
          i:   Integer;
          reg: TRegistry;
begin
  cmbFileType.ItemIndex:=0;
  cmbOSType.ItemIndex:=0;
  cmbLanguage.ItemIndex:=8;
  cmbCharSet.ItemIndex:=7;
  i:=300;
  SetLength(s, i);
  GetUserName(PChar(s), i);
  s:=PChar(s);
  txtAuthor.Text:=s;
  reg:=TRegistry.Create;
  try
    with reg do begin
      RootKey:=HKEY_LOCAL_MACHINE;
      OpenKey('\Software\Borland\Delphi\2.0', TRUE);
      s:=ReadString('RootDir');
      if (s<>'') and (s[Length(s)]='\') then
        Delete(s, Length(s), 1);
      txtCompiler.Text:=s+'\bin\brcc32.exe';
      CloseKey;
    end;
  finally
    reg.Free;
  end;
end;

procedure TfrmProperties.btnBuildClick(Sender: TObject);
var       hn:  TextFile;
          s:   String;
          ss:  String;
          sCD: String;
          StartupInfo: TStartupInfo;
          ProcessInfo: TProcessInformation;

          function ProperText(s: String): String;  // simply replace \ to /
          begin
            while Pos('\', s)>0 do
              s[Pos('\', s)]:='/';
            Result:=s;
          end;

begin
  sCD:=ExtractFilePath(TS.GetProjectName);
  if sCD<>'' then
    while sCD[Length(sCD)]='\' do
      Delete(sCD, Length(sCD), 1);
  AssignFile(hn, sCD+'\'+txtResource.Text);
  Rewrite(hn);
  Writeln(hn, '1 VERSIONINFO');
  Writeln(hn, 'FILEVERSION ', txtFileMaj1.Value, ',', txtFileMaj2.Value, ',', txtFileMin1.Value, ',', txtFileMin2.Value);
  Writeln(hn, 'PRODUCTVERSION ', txtProductMaj1.Value, ',', txtProductMaj2.Value, ',', txtProductMin1.Value, ',', txtProductMin2.Value);
  Writeln(hn, 'FILEOS ', cmbOSType.Items[cmbOSType.ItemIndex]);
  Writeln(hn, 'FILETYPE ', cmbFileType.Items[cmbFileType.ItemIndex]);
  Writeln(hn, '{');
  Writeln(hn, ' BLOCK "StringFileInfo"');
  Writeln(hn, ' {');
  Writeln(hn, '  BLOCK "040904E4"');
  Writeln(hn, '  {');
  Writeln(hn, '   VALUE "CompanyName", "', ProperText(txtCompanyName.Text), '\000"');
  Writeln(hn, '   VALUE "FileDescription", "', ProperText(txtDescription.Text), '\000"');
  Writeln(hn, '   VALUE "FileVersion", "', txtFileMaj1.Value, ',', txtFileMaj2.Value, ',', txtFileMin1.Value, ',', txtFileMin2.Value, '\000"');
  Writeln(hn, '   VALUE "ProductVersion", "', txtProductMaj1.Value, ', ', txtProductMaj2.Value, ',', txtProductMin1.Value, ',', txtProductMin2.Value, '\000"');
  if ProperText(txtInternalName.Text)<>'' then
    Writeln(hn, '   VALUE "InternalName", "', ProperText(txtInternalName.Text), '\000"');
  if ProperText(txtOriginalName.Text)<>'' then
    Writeln(hn, '   VALUE "OriginalFilename", "', ProperText(txtOriginalName.Text), '\000"');
  if ProperText(txtCopyRight.Text)<>'' then
    Writeln(hn, '   VALUE "LegalCopyright", "Copyright  ', ProperText(txtCopyRight.Text), '\000"');
  if ProperText(txtProductName.Text)<>'' then
    Writeln(hn, '   VALUE "ProductName", "', ProperText(txtProductName.Text), '\000"');
  if ProperText(txtTradeMark.Text)<>'' then
    Writeln(hn, '   VALUE "LegalTradeMarks", "', ProperText(txtTradeMark.Text), '\000"');
  if ProperText(txtComments.Text)<>'' then
    Writeln(hn, '   VALUE "Comments", "', ProperText(txtComments.Text), '\000"');
  if ProperText(txtFileFlags.Text)<>'' then
    Writeln(hn, '   VALUE "FileFlags", "', ProperText(txtFileFlags.Text), '\000"');
  if ProperText(txtAuthor.Text)<>'' then
    Writeln(hn, '   VALUE "Author", "', ProperText(txtAuthor.Text), '\000"');
  Write(  hn, '   VALUE "BuildType", "');
  if radDebugBuild.Checked then
    Write(hn, 'DebugBuild')
  else
    Write(hn, 'ReleaseBuild');
  if chkSpecialBuild.Checked then
    Write(hn, ', SpecialBuild');
  if chkInternalBuild.Checked then
    Writeln(hn, ', InternalBuild (NOT FOR RELEASE!)\000"')
  else
    Writeln(hn, '\000"');
  if chkIncludeDateStamp.Checked then
    Writeln(hn, '   VALUE "BuildDatestamp", "', DateTimeToStr(Now), '\000"');
  Writeln(hn, '  }');
  Writeln(hn, ' }');
  Writeln(hn, ' BLOCK "VarFileInfo"');
  Writeln(hn, ' {');
  s:=cmbLanguage.Items[cmbLanguage.ItemIndex];
  ss:=cmbCharSet.Items[cmbCharSet.ItemIndex];
  Writeln(hn, '  VALUE "Translation", 0x', Copy(s, 1, Pred(Pos(':', s))), ',', Copy(ss, 1, Pred(Pos(':', ss))));
  Writeln(hn, ' }');
  Writeln(hn, '}');

  s:=txtCompiler.Text;
  ss:='-fo'+sCD+'\'+ChangeFileExt(txtResource.Text, '.res')+' -w32 '+sCD+'\'+txtResource.Text;
  Flush(hn);
  CloseFile(hn);
  Sleep(500);
  FillChar(StartupInfo, SizeOf(TStartupInfo), 0);
  StartupInfo.cb:=SizeOf(TStartupInfo);
  StartupInfo.dwFlags:=STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow:=SW_HIDE;
  //ShowMessage('CommadnLine: '+ss);
  if not CreateProcess(PChar(s), PChar(ss), NIL, NIL, FALSE, CREATE_DEFAULT_ERROR_MODE, NIL, PChar(sCD), StartUpInfo, ProcessInfo) then
    ShowMessage('couldn''t run resource compiler!')
  else
  if chkAutoUpdateBuilding.Checked then
    if txtFileMin1.Value<txtFileMin1.MaxValue then
      txtFileMin1.Value:=txtFileMin1.Value+1
    else
    if txtFileMaj2.Value<txtFileMaj2.MaxValue then
      txtFileMaj2.Value:=txtFileMaj2.Value+1
    else
    if txtFileMaj1.Value<txtFileMaj1.MaxValue then
      txtFileMaj2.Value:=txtFileMaj2.Value+1;
end;

procedure TfrmProperties.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmProperties.btnSaveClick(Sender: TObject);
begin
  with TRegistry.Create do begin
    RootKey:=HKEY_CURRENT_USER;
    OpenKey('\Software\Abacus Research AG\VerExp', TRUE);
    WriteString('Compiler', txtCompiler.Text);
    WriteString('ResourceName', txtResource.Text);
    WriteString('CompanyName', txtCompanyName.Text);
    WriteString('Copyright', txtCopyRight.Text);
    WriteString('TradeMark', txtTradeMark.Text);
    WriteString('Author', txtAuthor.Text);
    WriteInteger('OSType', cmbOSType.ItemIndex);
    WriteInteger('Language', cmbLanguage.ItemIndex);
    WriteInteger('CharSet', cmbCharSet.ItemIndex);
    with TIniFile.Create(ChangeFileExt(TS.GetProjectName, '.DOF')) do begin
      WriteString('VerExp', 'RCFile', txtResource.Text);
      WriteInteger('VerExp', 'AutoUpdate', Ord(chkAutoUpdateBuilding.Checked));
      WriteInteger('VerExp', 'AutoUpdateSave', Ord(chkAutoUpdateSaving.Checked));
      Free;
    end;
    CloseKey;
    Free;
  end;
end;

procedure TfrmProperties.btnBrowseClick(Sender: TObject);
begin
  with TRegistry.Create do
  try
    RootKey:=HKEY_LOCAL_MACHINE;
    OpenKey('\Software\Borland\Delphi\2.0', TRUE);
    if ValueExists('RootDir') then
      dlgOpen.InitialDir:=ReadString('RootDir')+'\bin';
    CloseKey;
  finally
    Free;
  end;
  if dlgOpen.Execute then
    txtCompiler.Text:=dlgOpen.FileName;
end;

procedure TfrmProperties.btnLoadValuesClick(Sender: TObject);
begin
  LoadIni(Self);
end;

end.
