unit VerExpU;

interface

uses Windows, Controls, Forms, Menus, Dialogs, Classes, ToolIntf, EditIntf, ExptIntf,
     VirtIntf, Registry, SysUtils, Properties;

type
  TVerExpert       = class;

  TFileNotifier    = class;

  TPrjNotify       = class(TIAddInNotifier)
  public
    Form:            TfrmProperties;
    Expert:          TVerExpert;
    FileNotifier:    TFileNotifier;
    constructor      Create(Expert: TVerExpert; frm: TfrmProperties); virtual;
    procedure        FileNotification(NotifyCode: TFileNotification; const FileName: string; var Cancel: Boolean); override;
    procedure        LoadDefaults(sFile: string); virtual;
  end;

  TFileNotifier    = class(TIModuleNotifier)
  public
    Notifier:        TPrjNotify;
    constructor      Create(Notifier: TPrjNotify); virtual;
    procedure        Notify(NotifyCode: TNotifyCode); override;
    procedure        ComponentRenamed(ComponentHandle: Pointer; const OldName, NewName: string); override;
  end;

  TVerExpert       = class(TIExpert)
    menProperties:   TIMenuItemIntf;
    ProjectNotifier: TPrjNotify;
    frmProperties:   TfrmProperties;
    constructor      Create; virtual;
    destructor       Destroy; override;
    function         GetName: string; override;
    function         GetStyle: TExpertStyle; override;
    function         GetState: TExpertState; override;
    function         GetIDString: string; override;
    procedure        menPropertiesClick(Sender: TIMenuItemIntf);
    procedure        Execute; override;
  end;

function InitExpert(ToolServices: TIToolServices; Registerproc: TExpertRegisterproc; var Terminate: TExpertTerminateProc): Boolean; export; stdcall;

var expVer:         TVerExpert = NIL;

implementation

uses StdCode;

{var
  ToolServices: TIToolServices
}
//********* The Notifier Code *************************************************************************************************
constructor TFileNotifier.Create(Notifier: TPrjNotify);
begin
  inherited Create;
  Self.Notifier:=Notifier;
  //ShowMessage('FileNotifier created');
end;

procedure TFileNotifier.ComponentRenamed(ComponentHandle: Pointer; const OldName, NewName: string);
begin
  // Do nothing (interessiert mich vorerst so gut wie gar nicht ;-))
end;

procedure TFileNotifier.Notify(NotifyCode: TNotifyCode);
begin
  //ShowMessage('Notify '+IntToStr(Ord(NotifyCode)));
  if NotifyCode=ncAfterSave then
    ShowMessage('After Save');
  {
  TNotifyCode = (ncModuleDeleted, ncModuleRenamed, ncEditorModified,
    ncFormModified, ncEditorSelected, ncFormSelected, ncBeforeSave,
    ncAfterSave, ncFormSaving, ncProjResModified);
  }
end;

constructor TPrjNotify.Create(Expert: TVerExpert; frm: TfrmProperties);
begin
  inherited Create;
  //ShowMessage('ProjectNotifier created');
  Form:=frm;
  //ShowMessage('fORM.Name='+Form.Name);
  Self.Expert:=Expert;
  //ShowMessage('Expert='+Expert.ClassName);
end;

procedure   TPrjNotify.LoadDefaults(sFile: string);
var         hn:  TextFile;
            o:   String;
            s:   String;
            ss:  String;
            s1:  String;
            s2:  String;
            iCounter: Integer;
begin
  try
    AssignFile(hn, sFile);
    try
      Reset(hn);
      while not Eof(hn) do begin
        Readln(hn, o);
        ss:=Uppercase(o);
        with Form do begin
          if (Pos('FILEVERSION', ss)>0) and (Pos('"', ss)=0) then begin  // FileVersion
            s:=Copy(ss, Pos('FILEVERSION', ss)+11, Length(ss));
            while (s<>'') and (s[1]=' ') do
              Delete(s, 1, 1);
            txtFileMaj1.Value:=StrToIntDef(Copy(s, 1, Pred(Pos(',', s))), 1);
            while (s<>'') and (s[1]<>',') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            txtFileMaj2.Value:=StrToIntDef(Copy(s, 1, Pred(Pos(',', s))), 1);
            while (s<>'') and (s[1]<>',') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            txtFileMin1.Value:=StrToIntDef(Copy(s, 1, Pred(Pos(',', s))), 1);
            while (s<>'') and (s[1]<>',') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            txtFileMin2.Value:=StrToIntDef(s, 1);
          end;
          if (Pos('PRODUCTVERSION', ss)>0) and (Pos('"', ss)=0) then begin  // ProductVersion
            s:=Copy(ss, Pos('PRODUCTVERSION', ss)+14, Length(ss));
            while (ss<>'') and (ss[1]=' ') do
              Delete(s, 1, 1);
            txtProductMaj1.Value:=StrToIntDef(Copy(s, 1, Pred(Pos(',', s))), 1);
            while (s<>'') and (s[1]<>',') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            txtProductMaj2.Value:=StrToIntDef(Copy(s, 1, Pred(Pos(',', s))), 1);
            while (s<>'') and (s[1]<>',') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            txtProductMin1.Value:=StrToIntDef(Copy(s, 1, Pred(Pos(',', s))), 1);
            while (s<>'') and (s[1]<>',') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            txtProductMin2.Value:=StrToIntDef(s, 1);
          end;
          if Pos('FILEOS', ss)>0 then begin
            s:=Copy(ss, Pos('FILEOS', ss)+6, Length(ss));
            while (s<>'') and (s[1]=' ') do
              Delete(s, 1, 1);
            while (s<>'') and (s[Length(s)]=' ') do
              Delete(s, Length(s), 1);
            cmbOSType.ItemIndex:=Max(0, cmbOSType.Items.IndexOf(s));
          end;
          if Pos('FILETYPE', ss)>0 then begin
            s:=Copy(ss, Pos('FILETYPE', ss)+8, Length(ss));
            while (s<>'') and (s[1]=' ') do
              Delete(s, 1, 1);
            while (s<>'') and (s[Length(s)]=' ') do
              Delete(s, Length(s), 1);
            cmbFileType.ItemIndex:=Max(0, cmbFileType.Items.IndexOf(s));
          end;
          if Pos('COMPANYNAME', ss)>0 then begin
            s:=Copy(o, Pos('COMPANYNAME', ss)+11, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtCompanyName.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('FILEDESCRIPTION', ss)>0 then begin
            s:=Copy(o, Pos('FILEDESCRIPTION', ss)+15, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtDescription.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('INTERNALNAME', ss)>0 then begin
            s:=Copy(o, Pos('INTERNALNAME', ss)+12, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtInternalName.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('PRODUCTNAME', ss)>0 then begin
            s:=Copy(o, Pos('PRODUCTNAME', ss)+11, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtProductName.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('ORIGINALFILENAME', ss)>0 then begin
            s:=Copy(o, Pos('ORIGINALFILENAME', ss)+16, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtOriginalName.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('LEGALCOPYRIGHT', ss)>0 then begin
            s:=Copy(o, Pos('LEGALCOPYRIGHT', ss)+14, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            while Pos('COPYRIGHT  ', s)>0 do
              Delete(s, Pos('COPYRIGHT  ', s), 12);
            while Pos('COPYRIGHT', s)>0 do
              Delete(s, Pos('COPYRIGHT', s), 9);
            txtCopyright.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('LEGALTRADEMARK', ss)>0 then begin
            s:=Copy(o, Pos('LEGALTRADEMARK', ss)+14, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtTradeMark.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('COMMENTS', ss)>0 then begin
            s:=Copy(o, Pos('COMMENTS', ss)+8, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtComments.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('FILEFLAGS', ss)>0 then begin
            s:=Copy(o, Pos('FILEFLAGS', ss)+9, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtFileFlags.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('AUTHOR', ss)>0 then begin
            s:=Copy(o, Pos('AUTHOR', ss)+6, Length(ss));
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while (s<>'') and (s[1]<>'"') do
              Delete(s, 1, 1);
            Delete(s, 1, 1);
            while Pos('\000', s)>0 do
              Delete(s, Pos('\000', s), 4);
            txtAuthor.Text:=Copy(s, 1, Pred(Pos('"', s)));
          end;
          if Pos('BUILDTYPE', ss)>0 then begin
            radDebugBuild.Checked:=Pos('DebugBuild', ss)>0;
            radReleaseBuild.Checked:=Pos('ReleaseBuild', ss)>0;
            if not (radDebugBuild.Checked and radReleaseBuild.Checked) then
              radDebugBuild.Checked:=TRUE;
            chkSpecialBuild.Checked:=Pos('SpecialBuild', ss)>0;
            chkInternalBuild.Checked:=Pos('InternalBuild', ss)>0;
          end;
          chkIncludeDateStamp.Checked:=chkIncludeDateStamp.Checked or (Pos('BUILDDATESTAMP', ss)>0);
          if Pos('TRANSLATION', ss)>0 then begin
            s:=Copy(ss, Pos('TRANSLATION', ss)+11, Length(ss));
            while (s<>'') and (Copy(s, 1, 2)<>'0X') do
              Delete(s, 1, 1);
            Delete(s, 1, 2);
            s1:=Copy(s, 1, Pred(Pos(',', s)));
            s2:=Copy(s, Succ(Pos(',', s)), Length(s));
            while Pos(' ', s2)>0 do
              Delete(s2, Pos(' ', s2), 1);
            for iCounter:=0 to Pred(cmbLanguage.Items.Count) do
              if Pos(s1, cmbLanguage.Items[iCounter])>0 then begin
                cmbLanguage.ItemIndex:=iCounter;
                break;
              end;
            // end of for
            for iCounter:=0 to Pred(cmbCharSet.Items.Count) do
              if Pos(s2, cmbCharSet.Items[iCounter])>0 then begin
                cmbCharSet.ItemIndex:=iCounter;
                break;
              end;
            // end of for
          end;
        end;
      end;
    finally
      CloseFile(hn);
    end;
  except
    ToolServices.RaiseException('File has been currently locked!');
  end;
end;

procedure   TPrjNotify.FileNotification(NotifyCode: TFileNotification; const FileName: string; var Cancel: Boolean);
var         s: String;
            mModule: TIModuleInterface;
begin
  //ShowMessage(IntToStr(Ord(NotifyCode)));
  case NotifyCode of
    fnProjectOpened:  begin
                        //ShowMessage('Project Opened');
                        s:=ExtractOnlyFileName(FileName);
                        Form.txtInternalName.Text:=ChangeFileExt(s, '');
                        Form.txtOriginalName.Text:=FileName;
                        Form.txtProductName.Text:=FileName;
                        Form.Project:=FileName;

                        {
                        mModule:=ToolServices.GetModuleInterface(ExtractFileName(ToolServices.GetProjectName));
                        FileNotifier:=NIL;
                        FileNotifier:=TFileNotifier.Create(Self);
                        //ShowMessage(ExptIntf.ToolServices.GetCurrentFile);

                        if Assigned(mModule) then
                        try
                          mModule.AddNotifier(FileNotifier);
                          //ShowMessage('Filenotifier for '+ToolServices.GetProjectName+' created');
                        except
                          try
                            mModule.Release;
                          except
                            ToolServices.RaiseException('Couldn''t release Moduleinterface');
                          end;
                          ToolServices.RaiseException('FileHook couldn''t be created');
                        end;
                        }
                        if FileExists(ExtractFilePath(FileName)+'VERINFO.RC') then
                          LoadDefaults(ExtractFilePath(FileName)+'VERINFO.RC');
                      end;
    fnProjectClosing: begin
                        {
                        if Assigned(FileNotifier) then
                        try
                          FileNotifier.Free;
                        except
                          ExptIntf.ToolServices.RaiseException('Couldn''t remove Filenotifier');
                        end;
                        }
                      end;
  end;
end;

//********* ...and the Expert *************************************************************************************************
constructor TVerExpert.Create;
var         menMain:        TIMainMenuIntf;
            menViewWinList: TIMenuItemIntf;
            menViewMenu:    TIMenuItemIntf;
            reg:            TRegistry;
begin
  inherited Create;
  menMain:=ToolServices.GetMainMenu;
  if Assigned(menMain) then
  try
    menViewWinList:=menMain.FindMenuItem('EditSelectAll');
    if Assigned(menViewWinList) then
    try
      menViewMenu:=menViewWinList.GetParent;
      if Assigned(menViewMenu) then
      try
        menProperties:=menViewMenu.InsertItem(menViewWinList.GetIndex+1, '&Version Info...', 'VersionInfoItem', '', ShortCut(Ord('V'), [ssAlt, ssCtrl]), 0, 0, [mfEnabled, mfVisible], menPropertiesClick);
      finally
        menViewMenu.Free;
      end;
    finally
      menViewWinList.Free;
    end;
  finally
    menMain.Free;
  end;
  try
    try
      frmProperties:=TfrmProperties.Create(Application);
      with frmProperties do 
        RegisterToolServices(ExptIntf.ToolServices);
      try
        frmProperties.LoadIni(frmProperties);
        //ShowMessage('LoadINI successful!');
      finally
        //ShowMessage('VOR ProjectNotifierCreate');
        ProjectNotifier:=TPrjNotify.Create(Self, frmProperties);
        //ShowMessage('PrjNotifier wurde gerade errichtet');
        if not Assigned(ProjectNotifier) then
          //ShowMessage('couldn''t set notify hook')
        else
          ExptIntf.ToolServices.AddNotifier(ProjectNotifier);
      end;
    except
      ExptIntf.ToolServices.RaiseException('Couldn''t create Form');
    end;
  except
    ExptIntf.ToolServices.RaiseException('Couldn''t create Projectnotifier');
  end;
end;

destructor  TVerExpert.Destroy;
begin
  //ToolServices.RemoveNotifier(Hook);
  ProjectNotifier.Free;
  menProperties.Free;
  inherited Destroy;
end;

procedure   TVerExpert.Execute;
begin
  // do nothing (won't be called)
end;

function    TVerExpert.GetName: String;
begin
  Result:='VersionInfo Expert';
end;

procedure   TVerExpert.menPropertiesClick(Sender: TIMenuItemIntf);
begin
  frmProperties.Show;
end;

function    TVerExpert.GetStyle: TExpertStyle;
begin
  Result:=esAddIn;
end;

function    TVerExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function    TVerExpert.GetIDString: String;
begin
  Result:='Abacus.VersionInfoExpert';
end;

//******* Registration code ***************************************************************************************************
procedure Register;
begin
  RegisterLibraryExpert(TVerExpert.Create);
end;

procedure TerminateProc;
begin
end;

function    InitExpert(ToolServices: TIToolServices; Registerproc: TExpertRegisterproc; var Terminate: TExpertTerminateProc): Boolean; export; stdcall;
begin
  Result:=ExptIntf.ToolServices=NIL;
  if not Result then
    exit;
  ExptIntf.ToolServices:=ToolServices;
  if Assigned(ToolServices) then
    Application.Handle:=ToolServices.GetParentHandle;
  Terminate:=NIL;
  expVer:=TVerExpert.Create;
  RegisterProc(expVer);
end;

end.
